/**
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $id.java
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.engine.kpi;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.dom.DOMSource;

import org.apache.commons.io.FileUtils;
import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.notification.AsyncSender;
import org.ow2.petals.component.framework.api.notification.StatusListener;
import org.ow2.petals.component.framework.util.StringHelper;
import org.ow2.petals.component.framework.util.UtilFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

import com.ebmwebsourcing.wsstar.addressing.definition.WSAddressingFactory;
import com.ebmwebsourcing.wsstar.addressing.definition.api.EndpointReferenceType;
import com.ebmwebsourcing.wsstar.addressing.definition.api.ReferenceParametersType;
import com.ebmwebsourcing.wsstar.addressing.definition.api.WSAddressingException;
import com.ebmwebsourcing.wsstar.jaxb.notification.extension.types.ActionType;
import com.ebmwebsourcing.wsstar.notification.definition.WSNotificationFactory;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.WsnbConstants;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.FilterType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.QueryExpressionType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Subscribe;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.SubscribeResponse;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.SubscriptionPolicyType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.TopicExpressionType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Unsubscribe;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.UnsubscribeResponse;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.DestroyRegistrationResponse;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.RegisterPublisherResponse;
import com.ebmwebsourcing.wsstar.notification.definition.inout.WSNotificationWriter;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.extension.WSNotificationExtensionFactory;
import com.ebmwebsourcing.wsstar.notification.extension.api.ContextPolicyType;
import com.ebmwebsourcing.wsstar.notification.extension.api.ProcessPolicyType;
import com.ebmwebsourcing.wsstar.notification.extension.api.ResourcesUuidType;
import com.ebmwebsourcing.wsstar.notification.extension.api.SOAParameterType;
import com.ebmwebsourcing.wsstar.notification.extension.api.XsltStyleSheetType;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WsnSpecificTypeHelper;

public class KeyProcessIndicatorService {

    private File receivedUuidParentFolder;

    // TODO hard coded names, must be put in a shared constant (shared on all
    // componants that need it like se-notif, se-kpi, etc.)

    private static final QName NOTIFICATION_BROKER_INTERFACE_QNAME = new QName(
            "http://docs.oasis-open.org/wsn/brw-2", "NotificationBroker");

    private static final String NOTIFICATION_BROKER_SUBS_EP_NAME = "SubscriptionManagerServiceEndpoint";

    private static final String NOTIFICATION_BROKER_NS = "http://petals.ow2.org/petals-se-notification";

    private static final String NOTIFICATION_BROKER_SUBS_SERV_NAME = "SubscriptionManagerService";

    private static final QName NOTIFICATION_BROKER_SUBS_SERV_QNAME = new QName(
            NOTIFICATION_BROKER_NS, NOTIFICATION_BROKER_SUBS_SERV_NAME);

    private DeliveryChannel dc = null;

    private ComponentContext cptContext = null;

    private SOAParameterType consumerEndpoint = null;

    // List<String> susbcriptionsProcessedOnInit = null;
    private Map<String, EndpointReferenceType> returnedSubscriptionRefs = null;

    private Logger logger = null;

    private final String[] additionalNsAndPref;

    private AbstractComponent component;

    private List<AsyncSender> senders;

    public KeyProcessIndicatorService(final Logger logger, final DeliveryChannel deliveryChannel,
            final ComponentContext context, final AbstractComponent component,
            final SOAParameterType consumerEndpoint, final String[] additionalNsAndPref) {
        super();
        this.component = component;
        this.logger = logger;
        this.dc = deliveryChannel;
        this.cptContext = context;
        this.consumerEndpoint = consumerEndpoint;
        // Pattern id: EI_EXPOSE_REP2, type: EI2, category: MALICIOUS CODE: we
        // clone the result
        this.additionalNsAndPref = additionalNsAndPref.clone();
        this.returnedSubscriptionRefs = new ConcurrentHashMap<String, EndpointReferenceType>();
    }

    /**
     * Close the exchange
     * 
     * @param exchange
     *            The exchange to close
     */
    private void closeExchange(final MessageExchange exchange) {
        // close the messageExchange with the JBI NMR
        try {
            if (ExchangeStatus.ACTIVE.equals(exchange.getStatus())) {
                exchange.setStatus(ExchangeStatus.DONE);
                dc.send(exchange);
            }
        } catch (final MessagingException e) {
            final String errorMsg = "Error while closing JBI MessageExchange.";
            logger.log(Level.WARNING, errorMsg, e);
        }
    }

    /**
     * Get the returnedSubscriptionRefs
     * 
     * @return the returnedSubscriptionRefs
     */
    public Map<String, EndpointReferenceType> getReturnedSubscriptionRefs() {
        return this.returnedSubscriptionRefs;
    }

    // -------------------------------------------------------

    /**
     * Init Persistance folder for subscriptions uuid received by the
     * "KPI Service Engine". Create the folder if it has not already been
     * created.
     */
    public void init() {
        if (this.senders == null) {
            this.senders = new ArrayList<AsyncSender>();
        }
        this.receivedUuidParentFolder = new File(cptContext.getWorkspaceRoot() + File.separatorChar
                + "SaveReceivedSubscriptionUuid" + File.separatorChar);
        if (!receivedUuidParentFolder.exists()) {
            if (!receivedUuidParentFolder.mkdirs()) {
                if (logger != null) {
                    logger.warning("Unable to create the subscription response persistence folder");
                }
            }
        }
    }

    // ----- TDEJEAN : Multi threaded Persistence feature version -----
    // ----- NOTE : not tested yet !!!!
    /*
     * Init Persistance folder for subscriptions uuid received by the
     * "KPI Service Engine". Create the folder if it has not already been
     * created. tdejean : used for multi-threaded version
     */
    public void init(final String serviceUnitName) {
        final File receivedUuidSUFolder = new File(this.receivedUuidParentFolder.getAbsolutePath()
                + File.separatorChar + serviceUnitName + File.separatorChar);

        if (!receivedUuidSUFolder.exists()) {
            if (!receivedUuidSUFolder.mkdirs()) {
                if (logger != null) {
                    logger.warning("Unable to create the receive uuid SU folder for service Unit : \""
                            + serviceUnitName + "\"");
                }
            }
        }
    }

    // -------------------------------------- //

    // -------------------------------------------------------
    // ----------- Methods for Persistence feature -----------

    /**
     * persist a subscription uuid received in a subfolder whose name his the
     * service unit one
     * 
     * @param SUName
     *            the service unit name
     * @param subscriptionUuid
     *            the uuid to store (as an empty file whose name is the uuid)
     * @throws IOException
     * @throws WSAddressingException
     */
    private void persist(final EndpointReferenceType subscriptionRef, final String SUName,
            final String subscriptionUuid) throws IOException, WSAddressingException {

        final File serviceUnitPersistenceFolder = new File(
                receivedUuidParentFolder.getAbsolutePath() + File.separatorChar + SUName
                        + File.separatorChar);
        final File persistFolder = new File(serviceUnitPersistenceFolder, subscriptionUuid);
        if (!persistFolder.exists()) {
            if (!persistFolder.mkdirs()) {
                if (logger != null) {
                    logger.warning("Unable to create the receive uuid SU folder for service Unit : \""
                            + SUName + "\" and UUID \"" + subscriptionUuid + "\"");
                }
            }
        }

        final File persistFile = new File(persistFolder, "SubsReference.xml");
        persistFile.createNewFile();
        WSAddressingFactory.getInstance().newWSAddressingWriter()
                .writeEndpointReferenceType(subscriptionRef, persistFile);
    }

    /**
     * remove a given subscription uuid according to a given Service Unit name
     * 
     * @param SUName
     * @param uuid
     * @throws IOException
     */
    private void removeSubscriptionUuid(final String SUName, final String uuid) throws IOException {

        final File subscriptionToRemove = new File(receivedUuidParentFolder.getAbsolutePath()
                + File.separatorChar + SUName + File.separatorChar, uuid);

        if (subscriptionToRemove.exists()) {
            if (!FileUtils.deleteQuietly(subscriptionToRemove)) {
                throw new IOException(
                        "Deletion of subscription folder for subscription with uuid \"" + uuid
                                + "\" failed !");
            }
        }
    }

    /**
     * 
     * @return
     * @throws WSNotificationException
     * @throws WSAddressingException
     */
    public List<String> restoreSubscriptionRef(final String SUName) throws WSNotificationException,
            WSAddressingException {

        List<String> restoredSubsUuids = null;

        final File serviceUnitPersistenceFolder = new File(
                receivedUuidParentFolder.getAbsolutePath() + File.separatorChar + SUName
                        + File.separatorChar);

        if (serviceUnitPersistenceFolder.exists()) {

            restoredSubsUuids = new ArrayList<String>();

            final File[] persistedSubsriptionFiles = serviceUnitPersistenceFolder.listFiles();
            String uuid;
            for (final File persistedSubsriptionFile : persistedSubsriptionFiles) {
                uuid = persistedSubsriptionFile.getName();
                restoredSubsUuids.add(uuid);
                File subsRef = persistedSubsriptionFile.listFiles()[0];
                returnedSubscriptionRefs.put(uuid, WSAddressingFactory.getInstance()
                        .newWSAddressingReader().readEndpointReferenceType(subsRef));

            }
        }
        return restoredSubsUuids;
    }

    private Document sendRequest(final QName operation, final QName qService,
            final QName qInterface, final String endpoint, final Document request)
            throws MessagingException, PEtALSCDKException {
        Document result = null;

        InOut inOutExchange = null;
        // --- case 1 : unsubscribe request
        if (endpoint != null) {
            final ServiceEndpoint serviceEndpoint = cptContext.getEndpoint(qService, endpoint);
            if (serviceEndpoint == null) {
                throw new MessagingException("Endpoint not found for service " + qService.getLocalPart());
            } else {
                logger.log(Level.FINE,
                        "service endpoint registred :" + serviceEndpoint.getEndpointName());
                inOutExchange = dc.createExchangeFactory(serviceEndpoint).createInOutExchange();
            }
            // --- case 2 : subscribe request or unknown Uuids
        } else if (qInterface.getLocalPart().compareTo(
                NOTIFICATION_BROKER_INTERFACE_QNAME.getLocalPart()) == 0) {
            inOutExchange = dc.createExchangeFactory(qInterface).createInOutExchange();
        } else {
            throw new MessagingException("No endpoint jbi found to preform request '" + operation
                    + "' (Possible cause : bad resource uuid) !");
        }
        if (inOutExchange != null) {
            final NormalizedMessage normalizedMessage = inOutExchange.createMessage();
            normalizedMessage.setContent(new DOMSource(request));
            inOutExchange.setInMessage(normalizedMessage);
            inOutExchange.setOperation(operation);
            if (dc.sendSync(inOutExchange)) {
                if (inOutExchange.getOutMessage() != null) {
                    result = UtilFactory.getSourceUtil().createDocument(
                            inOutExchange.getOutMessage().getContent());
                } else if (inOutExchange.getFault() != null) {
                    result = UtilFactory.getSourceUtil().createDocument(
                            inOutExchange.getFault().getContent());
                }
            }
            closeExchange(inOutExchange);
        }

        return result;
    }

    /**
     * Get the subscribe payload
     * 
     * @param topicExprDialect
     * @param topicExprNamespace
     * @param topicExprPrefix
     * @param topicExprContent
     * @param msgContentXpathDialect
     * @param msgContentXpath
     * @param msgContentSOADialect
     * @param msgContentSOAServiceNamespace
     * @param msgContentSOAServiceName
     * @param msgContentSOAInterfaceNamespace
     * @param msgContentSOAInterfaceName
     * @param msgContentSOAEndpoint
     * @param xsltStyleSheet
     * @param actionType
     * @param contextPolicyCorrelationId
     * @param contextPolicyEndpoint
     * @param contextPolicyInterface
     * @param contextPolicyMeuuid
     * @param contextPolicyNotifDate
     * @param contextPolicyService
     * @param contextPolicyStatus
     * @return
     * @throws WSNotificationException
     * @throws WSAddressingException
     * @throws WSNotificationExtensionException
     * @throws ParserConfigurationException
     * @throws SAXException
     * @throws IOException
     */
    public Subscribe getSubscribePayload(final String topicExprDialect,
            final String topicExprNamespace, final String topicExprPrefix,
            final String topicExprContent, final String msgContentXpathDialect,
            final String msgContentXpath, final String msgContentSOADialect,
            final String msgContentSOAServiceNamespace, final String msgContentSOAServiceName,
            final String msgContentSOAInterfaceNamespace, final String msgContentSOAInterfaceName,
            final String msgContentSOAEndpoint, final InputStream xsltStyleSheet,
            final ActionType actionType, final Boolean contextPolicyCorrelationId,
            final Boolean contextPolicyEndpoint, final Boolean contextPolicyInterface,
            final Boolean contextPolicyMeuuid, final Boolean contextPolicyNotifDate,
            final Boolean contextPolicyService, final Boolean contextPolicyStatus)
            throws WSNotificationException, WSAddressingException,
            WSNotificationExtensionException, ParserConfigurationException, SAXException,
            IOException {
        final Subscribe subscribePayload = WSNotificationFactory.getInstance().createSubscribe();
        final EndpointReferenceType consumerEdpRef = WSAddressingFactory.getInstance()
                .newEndpointReferenceType();
        consumerEdpRef.setAddress(SOAParameterType.DEFAULT_PETALS_ADDRESS);
        final ReferenceParametersType ref = consumerEdpRef.newReferenceParameters();
        consumerEdpRef.setReferenceParameters(ref);
        WsnSpecificTypeHelper.setSOAParameter(consumerEndpoint, ref);

        subscribePayload.setConsumerReference(consumerEdpRef);

        final FilterType filter = WSNotificationFactory.getInstance().createFiltertype();

        final TopicExpressionType topic = WSNotificationFactory.getInstance()
                .createTopicExpressionType();

        topic.addTopicNameSpace(topicExprPrefix, topicExprNamespace);

        topic.setDialect(topicExprDialect);
        topic.setContent(topicExprContent);
        filter.setTopicExpression(topic);

        // MessageContent - Xpath
        if (msgContentXpathDialect != null) {
            final QueryExpressionType msgCtt = WSNotificationFactory.getInstance()
                    .createQueryExpressionType();
            msgCtt.setDialect(msgContentXpathDialect);
            // --- add xpath expression ---
            if (msgContentXpath != null) {
                WsnSpecificTypeHelper.setXpathExpressionsType(msgContentXpath, msgCtt);
            }
            // --- add service expression ---
            // ---------------------------
            filter.addMessageContent(msgCtt);
        }

        // MessageContent - SOAParam
        if (msgContentSOADialect != null) {
            final QueryExpressionType msgCtt = WSNotificationFactory.getInstance()
                    .createQueryExpressionType();
            msgCtt.setDialect(msgContentSOADialect);
            // --- add SOA Param

            final SOAParameterType soaVals = WSNotificationExtensionFactory.getInstance()
                    .createSOAParameterType();

            if (msgContentSOAServiceName != null) {
                QName service = null;
                if (msgContentSOAServiceNamespace != null) {
                    service = new QName(msgContentSOAServiceNamespace, msgContentSOAServiceName);
                } else {
                    service = new QName(msgContentSOAServiceName);
                }
                soaVals.setService(service);
            }

            if (msgContentSOAInterfaceName != null) {
                QName soaInterface = null;
                if (msgContentSOAInterfaceNamespace != null) {
                    soaInterface = new QName(msgContentSOAInterfaceNamespace,
                            msgContentSOAInterfaceName);
                } else {
                    soaInterface = new QName(msgContentSOAInterfaceName);
                }
                soaVals.setInterface(soaInterface);
            }

            if (msgContentSOAEndpoint != null) {
                soaVals.setEndpoint(msgContentSOAEndpoint);
            }

            WsnSpecificTypeHelper.setSOAParameter(soaVals, msgCtt);

            // --- add service expression ---
            // ---------------------------
            filter.addMessageContent(msgCtt);
        }

        // Filter
        subscribePayload.setFilter(filter);

        // ---- Add SubscriptionPolicy obj (xslt stylesheets) ---- :

        final SubscriptionPolicyType policy = WSNotificationFactory.getInstance()
                .createSubscriptionPolicyType();

        if (xsltStyleSheet != null) {
            final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            final DocumentBuilder builderForXslt = factory.newDocumentBuilder();

            final Document styleSheetAsDoc = builderForXslt.parse(xsltStyleSheet);
            final Element styleSheetAsElt = styleSheetAsDoc.getDocumentElement();

            final XsltStyleSheetType transformPolicy = WSNotificationExtensionFactory.getInstance()
                    .createXsltStyleSheetType();
            transformPolicy.setStyleSheet(styleSheetAsElt);

            WsnSpecificTypeHelper.setXsltStyleSheetType(transformPolicy, policy);
        }

        if (actionType != null) {
            final ProcessPolicyType processPolicy = WSNotificationExtensionFactory.getInstance()
                    .createProcessPolicyType();
            processPolicy.addAction(actionType);
            WsnSpecificTypeHelper.setProcessPolicyType(processPolicy, policy);
        }

        final ContextPolicyType contextPolicy = WSNotificationExtensionFactory.getInstance()
                .createContextPolicyType();

        contextPolicy.setCorrelationId(contextPolicyCorrelationId);
        contextPolicy.setEndpoint(contextPolicyEndpoint);
        contextPolicy.setInterface(contextPolicyInterface);
        contextPolicy.setMeuuid(contextPolicyMeuuid);
        contextPolicy.setNotifDate(contextPolicyNotifDate);
        contextPolicy.setService(contextPolicyService);
        contextPolicy.setStatus(contextPolicyStatus);

        WsnSpecificTypeHelper.setContextPolicyType(contextPolicy, policy);

        subscribePayload.setSubscriptionPolicy(policy);
        return subscribePayload;
    }

    public void unsubscribeOnStop(final String serviceUnitName, final String resourceUuid) {
        if (returnedSubscriptionRefs.size() > 0) {
            // convert RegisterPublisher Object to document representation
            try {
                final EndpointReferenceType subsRefItem = returnedSubscriptionRefs
                        .get(resourceUuid);

                // create Unsubscribe operation payload
                final Unsubscribe unsubscribePayload = WSNotificationFactory.getInstance()
                        .createUnsubscribe();

                final String externalAddress = subsRefItem.getAddress();

                final SOAParameterType itemSOAParam = WsnSpecificTypeHelper
                        .getSOAParameter(subsRefItem.getReferenceParameters());

                final String subscriptionManagerEdp = itemSOAParam.getEndpoint();
                final QName subscriptionManagerService = itemSOAParam.getService();
                final QName subscriptionManagerInterface = itemSOAParam.getInterface();

                final EndpointReferenceType edpRef = WSAddressingFactory.getInstance()
                        .newEndpointReferenceType();
                edpRef.setAddress(externalAddress);
                WsnSpecificTypeHelper.setEndpointReference(edpRef, unsubscribePayload);

                final String subsRefUuid = WsnSpecificTypeHelper
                        .getResourcesUuidType(subsRefItem.getReferenceParameters()).getUuids()
                        .get(0);
                final ResourcesUuidType rUuids = WSNotificationExtensionFactory.getInstance()
                        .createResourcesUuidType();
                rUuids.addUuid(subsRefUuid);

                WsnSpecificTypeHelper.setResourcesUuidType(rUuids, unsubscribePayload);

                // convert RegisterPublisher Object to document representation
                final Document requestAsDoc = WSNotificationWriter.getInstance(additionalNsAndPref)
                        .writeUnsubscribe(unsubscribePayload);
                sendRequest(WsnbConstants.UNSUBSCRIBE_QNAME, subscriptionManagerService,
                        subscriptionManagerInterface, subscriptionManagerEdp, requestAsDoc);
                returnedSubscriptionRefs.remove(resourceUuid);
                removeSubscriptionUuid(serviceUnitName, resourceUuid);
            } catch (final Exception e) {
                logger.warning("Can't unsubscribe on broker for SU \"" + serviceUnitName
                        + "\" and subs uuid \"" + resourceUuid + "\"");
            }
        }
    }

    /**
     * @param serviceUnitName
     * @param topicExprDialect
     * @param topicExprNamespace
     * @param topicExprPrefix
     * @param topicExprContent
     * @param msgContentXpathDialect
     * @param msgContentXpath
     * @param msgContentSOADialect
     * @param msgContentSOAServiceNamespace
     * @param msgContentSOAServiceName
     * @param msgContentSOAInterfaceNamespace
     * @param msgContentSOAInterfaceName
     * @param msgContentSOAEndpoint
     * @param xsltStyleSheet
     * @param actionType
     * @param contextPolicyCorrelationId
     * @param contextPolicyEndpoint
     * @param contextPolicyInterface
     * @param contextPolicyMeuuid
     * @param contextPolicyNotifDate
     * @param contextPolicyService
     * @param contextPolicyStatus
     * @param receivedUuid
     */
    public void asyncSubscribeOnStart(final String serviceUnitName, String topicExprDialect,
            String topicExprNamespace, String topicExprPrefix, String topicExprContent,
            String msgContentXpathDialect, String msgContentXpath, String msgContentSOADialect,
            String msgContentSOAServiceNamespace, String msgContentSOAServiceName,
            String msgContentSOAInterfaceNamespace, String msgContentSOAInterfaceName,
            String msgContentSOAEndpoint, InputStream xsltStyleSheet, ActionType actionType,
            Boolean contextPolicyCorrelationId, Boolean contextPolicyEndpoint,
            Boolean contextPolicyInterface, Boolean contextPolicyMeuuid,
            Boolean contextPolicyNotifDate, Boolean contextPolicyService,
            Boolean contextPolicyStatus, final List<String> receivedUuid) {
        try {

            final Subscribe subscribePayload = getSubscribePayload(topicExprDialect,
                    topicExprNamespace, topicExprPrefix, topicExprContent, msgContentXpathDialect,
                    msgContentXpath, msgContentSOADialect, msgContentSOAServiceNamespace,
                    msgContentSOAServiceName, msgContentSOAInterfaceNamespace,
                    msgContentSOAInterfaceName, msgContentSOAEndpoint, xsltStyleSheet, actionType,
                    contextPolicyCorrelationId, contextPolicyEndpoint, contextPolicyInterface,
                    contextPolicyMeuuid, contextPolicyNotifDate, contextPolicyService,
                    contextPolicyStatus);

            AsyncSender sender = this.component.getNotificationService().getAsyncSender(
                    new StatusListener() {

                        @Override
                        public void onUnsubscribed(UnsubscribeResponse response)
                                throws PEtALSCDKException {
                        }

                        @Override
                        public void onUnregistered(DestroyRegistrationResponse response)
                                throws PEtALSCDKException {
                        }

                        @Override
                        public void onSubscribed(SubscribeResponse response)
                                throws PEtALSCDKException {
                            if (response != null) {
                                try {
                                    final EndpointReferenceType returnedSubsRef = response
                                            .getSubscriptionReference();
                                    final String subscriptionId = WsnSpecificTypeHelper
                                            .getResourcesUuidType(
                                                    returnedSubsRef.getReferenceParameters())
                                            .getUuids().get(0);
                                    if (!StringHelper.isNullOrEmpty(subscriptionId)) {
                                        logger.log(Level.FINE, "subscriptionId returned : "
                                                + subscriptionId);

                                        returnedSubscriptionRefs.put(subscriptionId,
                                                returnedSubsRef);
                                        receivedUuid.add(subscriptionId);
                                        persist(returnedSubsRef, serviceUnitName, subscriptionId);
                                    }
                                } catch (Exception e) {
                                    throw new PEtALSCDKException(e);
                                }

                            }
                        }

                        @Override
                        public void onRegistered(RegisterPublisherResponse response)
                                throws PEtALSCDKException {
                        }

                        @Override
                        public void onException(Exception e) {
                            if (logger.isLoggable(Level.FINE)) {
                                logger.log(Level.WARNING,
                                        "Something is wrong in response processing", e);
                            } else {
                                logger.log(Level.WARNING, e.getMessage());
                            }
                        }
                    });

            sender.send(subscribePayload, NOTIFICATION_BROKER_SUBS_SERV_QNAME,
                    NOTIFICATION_BROKER_SUBS_EP_NAME);
            this.senders.add(sender);
        } catch (final Exception e) {
            final String msg = "An error occurs during 'initSubscriptions' proccess";
            if (this.logger.isLoggable(Level.FINE)) {
                this.logger.log(Level.FINE, msg, e);
            } else {
                this.logger.warning(msg);
            }
        }
    }

    /**
     * Stop current processes if any...
     * 
     * @throws IOException
     */
    public void stop() {
        if (this.senders != null) {
            for (AsyncSender sender : this.senders) {
                try {
                    sender.abort();
                    sender = null;
                } catch (PEtALSCDKException e) {
                    final String msg = "Problem while stopping the async sender";
                    if (this.logger.isLoggable(Level.FINE)) {
                        this.logger.log(Level.FINE, msg, e);
                    } else {
                        this.logger.warning(msg);
                    }
                }

            }
            this.senders.clear();
        }
    }

    /**
     * remove a given su persistence folder according to a given Service Unit
     * name
     * 
     * @param SUName
     */
    public void removeSUSubsPersistFolder(final String SUName) {

        final File folderToRemove = new File(receivedUuidParentFolder.getAbsolutePath()
                + File.separatorChar + SUName);

        if (folderToRemove.exists()) {
            if (!FileUtils.deleteQuietly(folderToRemove)) {
                this.logger.warning("Deletion of persistence folder for su with name \"" + SUName
                        + "\" failed !");
            }
        }
    }

}
