/**
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $id.java
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.engine.kpi.listener;

import java.util.List;
import java.util.logging.Level;

import javax.jbi.messaging.MessagingException;
import javax.xml.transform.dom.DOMSource;

import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.engine.kpi.ServiceEngine;
import org.ow2.petals.flowwatch.notification.Notification;
import org.ow2.petals.flowwatch.notification.NotificationReader;
import org.ow2.petals.flowwatch.utils.NotificationProcessingException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ebmwebsourcing.easycommons.xml.XMLPrettyPrinter;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.WsnbConstants;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Message;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.NotificationMessageHolderType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Notify;
import com.ebmwebsourcing.wsstar.notification.definition.inout.WSNotificationReader;
import com.ebmwebsourcing.wsstar.notification.definition.inout.WSNotificationWriter;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WsnSpecificTypeHelper;
import com.ebmwebsourcing.wsstar.notification.service.basenotification.WsnbNotificationConsumer;
import com.ebmwebsourcing.wsstar.notification.service.util.WSNotificationNotImplementedException;

/**
 * JBI listener for "key Process indicator" component
 * 
 * @author Thierry DÉJEAN (tdejean) - eBM WebSourcing
 * 
 */
public class JBIListener extends AbstractJBIListener implements WsnbNotificationConsumer {

    private NotificationReader notificationReader = null;

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.listener.AbstractListener#init()
     */
    @Override
    public void init() {
        super.init();
        this.notificationReader = new NotificationReader();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ebmwebsourcing.wsstar.notification.service.basenotification.
     * WsnbNotificationConsumer
     * #notify(com.ebmwebsourcing.wsstar.notification.base.Notify)
     */
    public void notify(final Notify request) throws WSNotificationException {
        Notification notification = null;
        try {
            // ------ Added for Debug --------
            if (this.getLogger().isLoggable(Level.FINE)) {
                this.getLogger().log(Level.FINE, "A notification from the Broker is received");
                final Document requestAsDoc = WSNotificationWriter.getInstance().writeNotify(
                        request);
                this.getLogger().log(
                        Level.FINE,
                        "\nNotification Request received From Broker :\n "
                        + XMLPrettyPrinter.prettyPrint(requestAsDoc));
            }
            // --------------------------------
            final List<NotificationMessageHolderType> listOfNotificationMessage = request
            .getNotificationMessage();
            for (final NotificationMessageHolderType notificationMessageHolderItem : listOfNotificationMessage) {
                final Message msg = notificationMessageHolderItem.getMessage();
                final Element msgContent = WsnSpecificTypeHelper.getContentFromMessage(msg);
                if (msgContent != null) {
                    if (this.getLogger().isLoggable(Level.FINE)) {
                        this.getLogger().log(
                                Level.FINE,
                                "NotificationMessageHolder content received :\n "
                                + XMLPrettyPrinter.prettyPrint(msgContent
                                        .getOwnerDocument()));
                    }
                    final javax.xml.transform.Source notifContentAsXmlSource = new DOMSource(
                            msgContent);
                    notification = this.notificationReader.read(notifContentAsXmlSource);
                    if (notification != null) {
                        ((ServiceEngine) this.component).getNotificationBlockingQueue().put(
                                notification);
                    }
                } else {
                    this
                    .getLogger()
                    .fine(
                            "NotificationMessageHolder content received is not \"ElementNSImpl\" type !:\n ");
                }
            }
        } catch (final NotificationProcessingException e) {
            throw new WSNotificationException(e);
        } catch (final InterruptedException e) {
            throw new WSNotificationException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.objectweb.petals.component.common.listener.AbstractJBIListener#
     * onJBIMessage
     * (org.objectweb.petals.component.common.util.MessageExchangeWrapper)
     */
    @Override
    public final boolean onJBIMessage(final Exchange exchange) {
        if (this.getLogger().isLoggable(Level.FINE)) {
            this.getLogger().log(Level.FINE, "\t\t ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
            this.getLogger().log(Level.FINE,
            "Notification Message received on \"onJBIMessage\" method. Nothing to do");
        }
        return true;
    }

    @Override
    public boolean onNotificationMessage(final Exchange notifExchange) {
        if (this.getLogger().isLoggable(Level.FINE)) {
            this.getLogger().log(Level.FINE, "\t\t ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
            this.getLogger().log(Level.FINE,
            "Notification Message received on \"onNotificationMessage\" method !");
        }
        return this.processMessageExchange(notifExchange);

    }

    private boolean processMessageExchange(final Exchange exchange) {
        try {
            // ============================================================
            // ========= NotificationConsumer interface operation =========
            // ============================================================
            if (WsnbConstants.NOTIFY_QNAME.getLocalPart().equalsIgnoreCase(
                    exchange.getOperationName())
                    && exchange.isInOnlyPattern()) {
                // Get in Message content
                final Document docContent = exchange.getInMessageContentAsDocument();
                this.notify(WSNotificationReader.getInstance().readNotify(docContent));
            }
            // ==================================================
            // ======= Others Operations and unknown ones =======
            // ==================================================
            else {
                exchange.setError(new Exception(
                "The KPI process only WS-Notification Notify on InOnly"));
            }
        } catch (final MessagingException e) {
            exchange.setError(new Exception(e));
        } catch (final WSNotificationNotImplementedException e) {
            exchange.setError(new Exception(e));
        } catch (final WSNotificationException e) {
            exchange.setError(new Exception(e));
        }
        return true;
    }
}
